# Growing up with AI: How can families play and learn with their new smart toys and companions?

By Stefania DrugaGraduate Student, Personal Robots GroupLEGO Papert Fellow 2018

I grew up in a small town in Transylvania, and will always remember the day when I assembled my first computer and loaded the first mp3 songs and movies on it. At the time we didn’t yet have Internet in my home, and I was exchanging files via CDs with my friends and my father’s colleagues.

By the time I went to college, my campus had its own intranet, and my friends and I built up a collection of hundreds of albums, television series, movies, software packages, anything you could imagine. It was as if all of the students were part of this giant web, seeding and peering on torrents while constantly exchanging the latest school news in forums and chat rooms. Later, I used the internet to find my first internship abroad, master’s scholarship, job, and apartment. I also founded a global community of young makers and hackers — [HacKIDemia](http://www.hackidemia.com/) — where kids could connect and exchange knowledge via videos, social groups, and local events: all thanks to this new medium that was breaking down geographic and institutional borders.

![](https://cdn-images-1.medium.com/max/1600/1*LEsnizUEIEL8zjwcbXGSjw.png)

I was part of the first wave of the internet generation, and now, looking back at my childhood, I realize how lucky this was. I grew up in a small town in a post-communist country and was the first in my family to go to college and study abroad; because of the internet, I had opportunities to connect and work with people from all over the world on meaningful and fun projects. Being a child that grew up with the web expanded my perspective on what is possible and it allowed me to participate in global conversations and projects while becoming a global citizen. After reflecting on my own childhood, I can’t help but wonder what this experience will look like for today’s children, who are not just growing up with the web, but are also the first generation of kids to grow up with artificial intelligence (AI) in their daily lives.

![](https://cdn-images-1.medium.com/max/1600/1*awsCdFIlLwZtTuB7URq3YA.png)

Recently, we’ve seen both the potential and the challenges of a global, connected community. While I was enchanted with the power of the web growing up, I now recognize its power through social media to overturn governments or manipulate large groups of people. We are finally starting to have the hard conversations around the ethics and policies that large companies and governments around the world need to start implementing in order to prevent future abuses. In this context I believe it is more crucial than ever to teach young people and their families to have a critical understanding of the new AI technologies and devices that are already part of their homes and lives.

New technologies are intimidating at first–the railroad, telegraph, automobiles, television, or personal computers all caused concern and even fear before they became commonplace. Young users drive adoption: kids just don’t have the same filters and fears as adults, and are more open to exploring what’s new. This fearlessness in turn inspires the adults around them to approach the new in more playful ways. There are ten million Alexa devices in the US alone, and Alexa for children was just released last week by Amazon. AI is in our lives, homes, and pockets and it’s safe to say it’s not going away.

> Providing kids with the tools and access to AI education will create a generation who are not simply passive consumers of this technology but, rather, active creators and shapers of its future.



As a Media Lab student, I believe that the best way to achieve critical understanding of something is to build or program it, and I want to provide families the option to do that with their smart toys and devices–and that’s why I created [Cognimates](http://cognimates.me/).

Cognimates is a platform based on the [Scratch](http://scratch.mit.edu/) programming language which provides a unique library of tools and activities for AI education. With it, parents and children (7–10 years old) participate in creative programming activities where they learn how to build games, program robots, and train their own AI models. Some of the activities are mediated by embodied intelligent agents, which help learners scaffold learning and collaborate more effectively.

![](https://cdn-images-1.medium.com/max/1600/1*NHREFCh3VLEBZp0cvxb4LA.png)

Empowering children to lead the way in the purposeful use of technology is not a new idea. In fact, the MIT Media Lab was co-founded by visionaries like Seymour Papert and Marvin Minsky, who both believed that young generations could inspire us to think and learn with and about technology in different ways. To do this, Papert created Logo, the first programming language for children (and a precursor of Scratch)–back when computers were still the size of an entire room and only available to a handful of researchers in academic centers. Minsky wrote [multiple essays ](https://web.media.mit.edu/~minsky/OLPC-5.html)inviting parents and schools to teach children how human learning and reasoning work by drawing parallels and lessons from cybernetics.

> Our ideas about psychology are still developing so rapidly that it wouldn’t make sense for us to select any current “theory of thinking” to teach. So instead, we’ll propose a different approach: to provide our children with ideas they could use to invent their own theories about themselves!—Marvin Minsky, OLPC Memo 5: Education and Psychology, 2009

![](https://cdn-images-1.medium.com/max/1600/1*UZu84q3Wm6H3xUQga5mb5w.jpeg)

![](https://cdn-images-1.medium.com/max/1600/1*BIf6VlJl6O8bXTDKOFbs4Q.jpeg)

Building on these ideas and the early work of Sherry Turkle and her seminal book Second Self—which explores how children could interrogate and explore their own nature while playing and interacting with “relational artifacts” (such as Tamagotchi and Real Baby doll)—I created the Cognimates platform. The name pays tribute to the work of Edith Ackermann; in her research, Ackermann called the toys with which children would engage “AniMates,” and she describes them as “playthings that do things.”

Before joining the Media Lab I designed[ STEAM education workshops](http://www.hackidemia.com/) and kits for children and parents, and while doing so noticed how schools, governments, and families became more supportive of maker education for children, seeing coding as a new form of literacy. The next frontier is artificial intelligence, which will completely change not only how we interact with and learn about technology, but also how we teach programming. If we want to tap into children’s natural fluency with exploring, modifying, and appropriating new technologies they grew up with, or [“kids power”](http://www.papert.org/articles/GhostInTheMachine.html) as Papert calls it, we need to find ways to allow them not only to consume but also to create and customize new AI games, devices, and toys.

![](https://cdn-images-1.medium.com/max/1600/1*gEcSWohYqXJeLcF6NCisbw.png)

So far we’ve been testing Cognimates with more than 150 children in four countries since I built the first version of the platform in beginning of the year. Ever since I also recruited a very talented and passionate team of undergraduate students to help with platform development and family workshops. Based on our ongoing studies in schools and and after schools programs in the Greater Boston Area we created a series of starter projects and learning guides which are currently available on our website.

I will share below a couple of examples of our projects and activities while underlying what key concepts children are learning from them.

![](https://cdn-images-1.medium.com/max/1600/1*mgg9XAHMHvIhS7DoBmwEkg.jpeg)

**Program games that learn over time**



Children already love to play games and create their own games in Scratch. With our Cognimates Extensions we allow them to build games that learn over time as they are playing them. For example our workshop participants trained a Rock Paper Scissors game to recognize their gestures. Everytime the game makes a mistake the children can teach it what the correct answer was and make the game smarter over time. This allows them to learn how computer vision works and supervised learning works and how they could teach the computer or a robot to recognize a specific object from a collection of things and correct its errors such that it becomes better over time.

**Create customized chatbots**

Most of the children we’ve been working with are already familiar with chatbots and conversational agents like Siri and Alexa but they wanted to create a chatbot that would be able to pick-up on more subtle messages such as backhanded compliments. So they built a game where a dog would jump around and act happy if they would tell it a happy message or act confused and inquisitive if they would give it a fake compliment like “you are kind of brave.”

![](https://cdn-images-1.medium.com/max/1600/1*U0nfjdqMPh3xb4Kqv9UeIQ.jpeg)

**Program your smart home or classroom**

Many of our extensions allow children to start programing IoT devices like HueLights or Wemo Plugs and teach smart assistants like Alexa how to answer specific questions, leave messages for other people, or run code via voice. As many of these devices already exist in the homes, we wanted to provide children a way to customize and control these devices and create immersive, whimsical, and magical experiences such as the “Alice in Wonderland” project in the following video.



**A global collaborative community**

Our goal is to enable children and parents from various communities around the world to collaborate and share their Cognimates projects. Up to now we have organized workshops in the USA, Germany, Denmark, and Saudi Arabia, with upcoming festivals and teacher trainings in Chile, Italy, and Sweden.

We invite you to try out our [starter projects](https://mitmedialab.github.io/cognimates-website/projects/), make your own projects, and [share them with us](http://mailto:cognimates@gmail.com/). We will continue to expand our gallery with community projects and learning guides. If you would like to invite us to your school, museum or community center, [please contact us ](http://mailto:cognimates@gmail.com/)and join our[ mailing list.](http://eepurl.com/dunHMH)

![](https://cdn-images-1.medium.com/max/1600/1*icjNWqIy1mogOL5nmGCapQ.jpeg)

![](https://cdn-images-1.medium.com/max/1600/1*7IBWG1CCxd1_OevUFF9jDw.jpeg)

### Acknowledgements

First and foremost I would like to thank my advisor Cynthia Breazeal for her belief in and support of this project, for mobilizing resources and challenging my thinking around different ways for children to learn about AI; and for pushing me to think bigger and broader to democratize AI for all. I would also like to thank the Cognimates team for putting their soul and hard work into this project and for being there with me debugging late into the night and sharing laughter and jokes with our students. I also want to thank LEGO Foundation for awarding me the Lego Papert Fellowship, and NTT Data for funding a SIG, that supported my research and part of the travel for this project.

![](https://cdn-images-1.medium.com/max/1600/1*_dJBpdAmyDnkYPhV_anOrw.jpeg)

We are also very grateful to the following schools and organizations that are working with us for providing Cognimates workshops:

* East Somerville Community Schools, Cambridge MA: Arnulfo Reyes, Community Schools Site Coordinator

* Empow Studios, Lexington, MA: Matthew Silverstein, Director of Program Operations

* Shady Hill School, Cambridge, MA: Francesco Cupolo, Assistant Director of Co-Curricular Programs

* Redi School Berlin, Germany: Pierluigi Delgiudice, Project Manager–Kids Program

* International School of Billund: Camilla Uhre Fog, Head of School

I also want to thank the Lifelong Kindergarten group at the Media Lab for creating Scratch and for making it open source. Finally, thanks to the parents and children who have participated in our workshops and studies so far; I look forward to see what future AI programs and games you will make!

![](https://cdn-images-1.medium.com/max/1600/1*XNIwjdkQVTYSJGnN6uOLMg.png)

